package info.textgrid.middleware.test.online.tg;

import java.io.IOException;
import java.util.Properties;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import org.apache.cxf.jaxrs.client.Client;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import info.textgrid.middleware.test.online.OAIPMHUtilitiesOnline;

/**
 * <p>
 * Some online tests for the TextGrid OAIMPH service --> verb=ListRecords <--
 * </p>
 * 
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2022-09-26
 * @since 2022-09-12
 */
@Ignore
public class TestTGListRecordsOnline {

  // **
  // STATICS
  // **

  // Some JAXRS things.
  private static String oaipmhEndpoint;
  static Client oaipmhWebClient;

  // Properties
  private static String checkListRecordsDCSet;
  private static String checkListRecordsDCFrom;
  private static String checkListRecordsDCUntil;
  private static String checkListRecordsDATACITEFrom;
  private static String checkListRecordsDATACITEUntil;
  private static String checkListRecordsIDIOMFrom;
  private static String checkListRecordsIDIOMUntil;

  // **
  // PREPARATIONS
  // **

  /**
   * @throws Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception {

    // Get properties.
    Properties p =
        OAIPMHUtilitiesOnline.getPropertiesFromFile(OAIPMHUtilitiesOnline.PROPERTIES_FILE);

    // Set properties.
    oaipmhEndpoint = p.getProperty("oaipmhEndpoint");

    checkListRecordsDCSet = p.getProperty("checkListRecordsDCSet");
    checkListRecordsDCFrom = p.getProperty("checkListRecordsDCFrom");
    checkListRecordsDCUntil = p.getProperty("checkListRecordsDCUntil");

    checkListRecordsDATACITEFrom = p.getProperty("checkListRecordsDATACITEFrom");
    checkListRecordsDATACITEUntil = p.getProperty("checkListRecordsDATACITEUntil");

    checkListRecordsIDIOMFrom = p.getProperty("checkListRecordsIDIOMFrom");
    checkListRecordsIDIOMUntil = p.getProperty("checkListRecordsIDIOMUntil");

    // Get web client from endpoint.
    oaipmhWebClient = OAIPMHUtilitiesOnline.getOAIPMHWEebClient(oaipmhEndpoint);
  }

  /**
   * @throws Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception {
    //
  }

  /**
   * @throws Exception
   */
  @Before
  public void setUp() throws Exception {
    //
  }

  /**
   * @throws Exception
   */
  @After
  public void tearDown() throws Exception {
    //
  }

  // **
  // TESTS
  // **

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsOAIDCMorePages() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsOAIDCMorePages()");

    OAIPMHUtilitiesOnline.examineTGList(oaipmhWebClient,
        OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.NO_SET,
        OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
        18, 100,
        OAIPMHUtilitiesOnline.NO_THREAD_NAME,
        OAIPMHUtilitiesOnline.NO_FROM,
        OAIPMHUtilitiesOnline.NO_UNTIL,
        OAIPMHUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK);

    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsOAIDCSetMorePages() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsOAIDCSetMorePages()");

    OAIPMHUtilitiesOnline.examineTGList(oaipmhWebClient,
        OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        checkListRecordsDCSet,
        OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
        20, 100,
        OAIPMHUtilitiesOnline.NO_THREAD_NAME,
        OAIPMHUtilitiesOnline.NO_FROM,
        OAIPMHUtilitiesOnline.NO_UNTIL,
        OAIPMHUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK);

    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  @Ignore
  public void testListRecordsOAIDCAllPages() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsOAIDCAllPages()");

    OAIPMHUtilitiesOnline.examineTGList(oaipmhWebClient,
        OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.NO_SET,
        OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
        0, 100,
        OAIPMHUtilitiesOnline.NO_THREAD_NAME,
        OAIPMHUtilitiesOnline.NO_FROM,
        OAIPMHUtilitiesOnline.NO_UNTIL,
        OAIPMHUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK);

    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsOAIDCMorePagesFromUntil() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsOAIDCMorePagesFromUntil()");

    OAIPMHUtilitiesOnline.examineTGList(oaipmhWebClient,
        OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        checkListRecordsDCSet,
        OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
        0, 100,
        OAIPMHUtilitiesOnline.NO_THREAD_NAME,
        checkListRecordsDCFrom,
        checkListRecordsDCUntil,
        OAIPMHUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK);

    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test(expected = IOException.class)
  public void testListRecordsIDIOMMETSSomePagesAndMetadataFormatWithRestok() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING
        + "testListRecordsIDIOMMETSSomePagesAndMetadataFormatWithRestok()");

    OAIPMHUtilitiesOnline.examineTGList(oaipmhWebClient,
        OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.NO_SET,
        OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
        3, 30,
        OAIPMHUtilitiesOnline.NO_THREAD_NAME,
        OAIPMHUtilitiesOnline.NO_FROM,
        OAIPMHUtilitiesOnline.NO_UNTIL,
        OAIPMHUtilitiesOnline.METADATA_FORMAT_WITH_RESTOK);

    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsIDIOMMETSAllPages() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsIDIOMMETSAllPages()");

    if (OAIPMHUtilitiesOnline.TEST_ALL_PAGES) {

      OAIPMHUtilitiesOnline.examineTGList(oaipmhWebClient,
          OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
          OAIPMHUtilitiesOnline.NO_SET,
          OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
          0, 30,
          OAIPMHUtilitiesOnline.NO_THREAD_NAME,
          OAIPMHUtilitiesOnline.NO_FROM,
          OAIPMHUtilitiesOnline.NO_UNTIL,
          OAIPMHUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK);

      System.out.println(OAIPMHUtilitiesOnline.OK);
    } else {
      System.out.println("...skipping");
    }
  }

  /**
   * NOTE The amount of records will change due to changes on the data! Please check from and until
   * values before!
   *
   * @throws IOException
   */
  @Test
  public void testListRecordsIDIOMMETSMorePagesFromUntil() throws IOException {

    System.out
        .println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsIDIOMMETSMorePagesFromUntil()");

    OAIPMHUtilitiesOnline.examineTGList(oaipmhWebClient,
        OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.NO_SET,
        OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
        6, 30,
        OAIPMHUtilitiesOnline.NO_THREAD_NAME,
        checkListRecordsIDIOMFrom,
        checkListRecordsIDIOMUntil,
        OAIPMHUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK);

    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsIDIOMMETSMorePages() throws IOException {

    System.out
        .println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsIDIOMMETSMorePages()");

    OAIPMHUtilitiesOnline.examineTGList(oaipmhWebClient,
        OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.NO_SET,
        OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
        6, 30,
        OAIPMHUtilitiesOnline.NO_THREAD_NAME,
        OAIPMHUtilitiesOnline.NO_FROM,
        OAIPMHUtilitiesOnline.NO_UNTIL,
        OAIPMHUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK);

    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsDATACITEMorePages() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsDATACITEMorePages()");

    OAIPMHUtilitiesOnline.examineTGList(oaipmhWebClient,
        OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.NO_SET,
        OAIPMHUtilitiesOnline.OAI_DATACITE_PREFIX,
        15, 100,
        OAIPMHUtilitiesOnline.NO_THREAD_NAME,
        OAIPMHUtilitiesOnline.NO_FROM,
        OAIPMHUtilitiesOnline.NO_UNTIL,
        OAIPMHUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK);

    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsDATACITEAllPages() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsDATACITEAllPages()");

    if (OAIPMHUtilitiesOnline.TEST_ALL_PAGES) {
      OAIPMHUtilitiesOnline.examineTGList(oaipmhWebClient,
          OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
          OAIPMHUtilitiesOnline.NO_SET,
          OAIPMHUtilitiesOnline.OAI_DATACITE_PREFIX,
          0, 100,
          OAIPMHUtilitiesOnline.NO_THREAD_NAME,
          OAIPMHUtilitiesOnline.NO_FROM,
          OAIPMHUtilitiesOnline.NO_UNTIL,
          OAIPMHUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK);

      System.out.println(OAIPMHUtilitiesOnline.OK);
    } else {
      System.out.println("..skipping");
    }
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsDATACITEMorePagesFromUntil() throws IOException {

    System.out
        .println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsDATACITEMorePagesFromUntil()");

    OAIPMHUtilitiesOnline.examineTGList(oaipmhWebClient,
        OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.NO_SET,
        OAIPMHUtilitiesOnline.OAI_DATACITE_PREFIX,
        15, 100,
        OAIPMHUtilitiesOnline.NO_THREAD_NAME,
        checkListRecordsDATACITEFrom,
        checkListRecordsDATACITEUntil,
        OAIPMHUtilitiesOnline.NO_METADATA_FORMAT_WITH_RESTOK);

    System.out.println(OAIPMHUtilitiesOnline.OK);
  }

  /**
   * @throws IOException
   * @throws ExecutionException
   * @throws InterruptedException
   */
  @Test
  public void testListRecordsConcurrentlyIDIOMMETSMorePages()
      throws IOException, InterruptedException, ExecutionException {

    System.out
        .println(
            OAIPMHUtilitiesOnline.TESTING + "testListRecordsConcurrentlyIDIOMMETSMorePages()");

    ExecutorService executor = Executors.newFixedThreadPool(3);

    Future<Boolean> f1 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET, OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX, 2, 30, "D1"));
    Future<Boolean> f2 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET, OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX, 3, 30, "D2"));
    Future<Boolean> f3 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET, OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX, 4, 30, "D3"));

    executor.shutdown();

    System.out.println(OAIPMHUtilitiesOnline.OK + ": [D1]=" + f1.get() + ", [D2]=" + f2.get()
        + ", [D3]=" + f3.get());
  }


  /**
   * @throws InterruptedException
   * @throws ExecutionException
   */
  @Test
  public void testRestokConcurrentlyListRecordsDC()
      throws InterruptedException, ExecutionException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testRestokConcurrentlyListRecordsDC()");

    ExecutorService executor = Executors.newFixedThreadPool(4);

    Future<Boolean> f1 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET, OAIPMHUtilitiesOnline.OAI_DC_PREFIX, 44, 100, "DC1"));
    Future<Boolean> f2 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET, OAIPMHUtilitiesOnline.OAI_DC_PREFIX, 33, 100, "DC2"));
    Future<Boolean> f3 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET, OAIPMHUtilitiesOnline.OAI_DC_PREFIX, 66, 100, "DC3"));
    Future<Boolean> f4 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET, OAIPMHUtilitiesOnline.OAI_DC_PREFIX, 28, 100, "DC4"));

    executor.shutdown();

    System.out.println(
        OAIPMHUtilitiesOnline.OK + ": [DC1]=" + f1.get() + ", [DC2]=" + f2.get() + ", [DC3]="
            + f3.get() + ", [DC4]=" + f4.get());
  }

  /**
   * @throws IOException
   * @throws InterruptedException
   * @throws ExecutionException
   */
  @Test
  public void testRestokConcurrentlyListRecords() throws InterruptedException, ExecutionException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testRestokConcurrentlyListRecords()");

    ExecutorService executor = Executors.newFixedThreadPool(3);

    Future<Boolean> f1 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET, OAIPMHUtilitiesOnline.OAI_DC_PREFIX, 6, 100, "B1"));
    Future<Boolean> f2 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET, OAIPMHUtilitiesOnline.OAI_DC_PREFIX, 7, 100, "B2"));
    Future<Boolean> f3 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET, OAIPMHUtilitiesOnline.OAI_DC_PREFIX, 8, 100, "B3"));
    Future<Boolean> f4 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET, OAIPMHUtilitiesOnline.OAI_DC_PREFIX, 9, 100, "B4"));

    executor.shutdown();

    System.out.println(OAIPMHUtilitiesOnline.OK + ": [B1]=" + f1.get() + ", [B2]=" + f2.get()
        + ", [B3]=" + f3.get() + ", [B4]=" + f4.get());
  }

  /**
   * @throws InterruptedException
   * @throws ExecutionException
   */
  @Test
  public void testListRecordsRestokConcurrentlyDCAndIDIOMMETS()
      throws InterruptedException, ExecutionException {

    System.out.println(
        OAIPMHUtilitiesOnline.TESTING + "testListRecordsRestokConcurrentlyDCAndIDIOMMets()");

    ExecutorService executor = Executors.newFixedThreadPool(3);

    Future<Boolean> f1 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient,
            OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET,
            OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            34, 100,
            "DC1"));
    Future<Boolean> f2 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient,
            OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET,
            OAIPMHUtilitiesOnline.OAI_DC_PREFIX,
            27, 100,
            "DC2"));
    Future<Boolean> f3 = executor.submit(
        new TGOAIPMHResumptionTokenThread(oaipmhWebClient,
            OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.NO_SET,
            OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX,
            5, 30,
            "IDIOM"));

    executor.shutdown();

    System.out.println(OAIPMHUtilitiesOnline.OK + ": [DC1]=" + f1.get() + ", [DC2]=" + f2.get()
        + ", [IOIOM]=" + f3.get());
  }

}
